/*
     This file is part of GNUnet

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/
/**
 * This is the main file for the gtk+ client.
 * @file src/gtkui/main.c
 * @author Christian Grothoff
 * @author Igor Wronsky
 **/

#include "config.h"
#include <gtk/gtk.h>
#include <glib.h>
#include <unistd.h>
#include <sys/types.h>

#include "util/getopt.h"
#include "util/storage.h"
#include "util/logging.h"
#include "gtkui/helper.h"
#include "gtkui/search.h"
#include "gtkui/insert.h"
#include "textui/requestmanager.h"
#include "textui/searchutil.h"
#include "textui/insertfile.h"
#include "configuration.h"
#include "knownhosts.h"

static GtkWidget * main_window_input_line;
static GtkNotebook * notebook;

static void close_search( GtkButton   *button,
                  gpointer dummy );
static void about(GtkWidget *dummy, gpointer data);

static char *aboutString[]=
  { 
    "\nGNUnet ",
	GNUNET_VERSION,
	" (",
	GNUNET_BRANCH,
	") gnunet-gtk ",
	GNUNET_GTKUI_VERSION,
	"\n\n\n",
	"This is free software, released under GNU General Public license version 2.",
	"\n\n\n",
    "For more information, visit GNUnet homepage at \n\n",
	"http://www.ovmj.org/GNUnet/\n"
  };

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",         NULL,         NULL, 0, "<Branch>" },
  { "/File/_Insert",   "<control>I", openSelectFile, 0, NULL },
  { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },
  { "/File/_Abort search",         "<control>A", close_search, 0, NULL },
  { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },
  { "/File/_Quit",     "<control>Q", gtk_main_quit, 0, NULL },
  { "/_Options",      NULL,         NULL, 0, "<Branch>" },
  { "/Options/Preferences(not impl)",  NULL,         NULL, 0, NULL },
  { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
  { "/_Help/About",   NULL,         about, 0, NULL },
};

/**
 * This displays an about window
 *
 */
static void about(GtkWidget *dummy, gpointer data)
{
  GtkWidget *window, *box1, *table, *text, *button;
  int i;

  /* create about window */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
  gtk_window_set_title(GTK_WINDOW(window), "About gnunet-gtk");
  gtk_widget_set_usize(GTK_WIDGET(window), 600, 300);

  box1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER (window), box1);
  gtk_widget_show(box1);

  table = gtk_table_new(2, 2, FALSE);
  gtk_table_set_row_spacing(GTK_TABLE (table), 0, 2);
  gtk_table_set_col_spacing(GTK_TABLE (table), 0, 2);
  gtk_box_pack_start(GTK_BOX (box1), table, TRUE, TRUE, 0);
  gtk_widget_show(table);

  /* create a text widget */
  text = gtk_text_new(NULL, NULL);
  gtk_text_set_editable(GTK_TEXT (text), FALSE);
  gtk_table_attach(GTK_TABLE (table), text, 0, 1, 0, 1,
            GTK_EXPAND | GTK_SHRINK | GTK_FILL,
            GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_widget_show(text);
  gtk_widget_realize(text);

  /* write some about text */
  gtk_text_freeze(GTK_TEXT (text));

  for(i = 0;i<sizeof(aboutString)/sizeof(char *); i++) {
    gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL,
	aboutString[i], -1);
  }
  
  gtk_text_thaw(GTK_TEXT(text));

  /* finish with a close button */
  button = gtk_button_new_with_label("Right");
  gtk_box_pack_start(GTK_BOX (box1), button, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
              GTK_SIGNAL_FUNC(destroyWidget), window);
  gtk_widget_show(button);
  gtk_widget_show(window);
}

/**
 * This method is called whenever the user clicks the
 * search button of the main window. data contains
 * the notebook where the opened result window will be put.
 *
 */
static void search(GtkWidget * widget,
		   GtkNotebook * notebook) {
  gchar * searchString;
  GtkWidget * label, * tmp;
 
  searchString 
    = gtk_entry_get_text(GTK_ENTRY(main_window_input_line));

  tmp=getSearchWindow(searchString);
  if(tmp!=NULL) {
    label = gtk_label_new(searchString);
    gtk_notebook_append_page(notebook, tmp, label);
    gtk_widget_show(tmp);
  }

  gtk_entry_set_text(GTK_ENTRY(main_window_input_line),"");
}


/* exit the application */
static void destroy(GtkWidget * widget,
		    gpointer data) {
  gtk_main_quit();
}

/**
 * Remove the active page from the notebook.
 * The respective search will be stopped as well
 * (by a callback assigned to the page earlier on).
 *
 */
static void close_search(GtkButton * button,
                  gpointer dummy)
{
    gint page;

    page = gtk_notebook_get_current_page(notebook);
    gtk_notebook_remove_page(notebook, page);
    /* Need to refresh the widget --
     This forces the widget to redraw itself. */
    gtk_widget_draw(GTK_WIDGET(notebook), NULL);
}

/**
 * This creates the main window
 *
 */
static void makeMainWindow() {
  GtkWidget * window;
  GtkWidget * button;
  GtkWidget * hbox, * vbox;
  GtkWidget * menubar;
  GtkWidget * table;
  GtkWidget * label;
  GtkAccelGroup * accel_group;
  GtkItemFactory * item_factory;
  gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
  gtk_window_set_title(GTK_WINDOW(window), "GNUnet: gtk+ GUI");
  gtk_widget_set_usize(GTK_WIDGET(window), 800, 300);
  vbox = gtk_vbox_new(FALSE, 1);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  gtk_signal_connect(GTK_OBJECT(window), 
		      "delete_event",
		      GTK_SIGNAL_FUNC(deleteEvent), NULL);
  gtk_signal_connect(GTK_OBJECT(window), 
		      "destroy",
		      GTK_SIGNAL_FUNC(destroy), NULL);
  gtk_widget_show(vbox);
  
  /* create pulldown menues */
  accel_group = gtk_accel_group_new ();
  item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>",
                       accel_group);
  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
  gtk_window_add_accel_group(GTK_WINDOW (window), accel_group);
  menubar = gtk_item_factory_get_widget(item_factory, "<main>");
  gtk_box_pack_start(GTK_BOX (vbox), menubar, FALSE, TRUE, 0);
  gtk_widget_show(menubar);

  /* a table to put the notebook to */
  table = gtk_table_new(6,6,TRUE);
  gtk_box_pack_start(GTK_BOX (vbox), table, TRUE, TRUE, 0);
  gtk_widget_show(table);
  
  /* add "notebook" for search results */
  notebook = GTK_NOTEBOOK(gtk_notebook_new());
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
  gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(notebook), 0,6,0,6);
  gtk_widget_show(GTK_WIDGET(notebook));

  /* At the bottom, put the search box */
  hbox=gtk_hbox_new(FALSE, 1);
  gtk_box_pack_start(GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(hbox);
 
  /* search entry label */
  label = gtk_label_new("Keyword(s):");
  gtk_box_pack_start(GTK_BOX(hbox),
  			label, FALSE, FALSE, 0);
  gtk_widget_show(label);

  /* search entry */
  main_window_input_line = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), 
		     main_window_input_line, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(main_window_input_line),"activate",
                     GTK_SIGNAL_FUNC(search),
                     notebook);
  gtk_widget_show(main_window_input_line);

  /* search button */
  button = gtk_button_new_with_label("Search");
  gtk_signal_connect(GTK_OBJECT (button), 
		      "clicked",
		      GTK_SIGNAL_FUNC(search), notebook);
  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
  gtk_widget_show(button);

  /* show the main window */
  gtk_widget_show(window);
}

/**
 * Print a list of the options we offer.
 **/
static void printhelp() {
  printf("USAGE: gnunet-gtk [OPTIONS]\n");
  printf("-h, --help                    : print this page\n");
  printf("-c FILENAME, --config=FILENAME: load config file (defaults: %s)\n",
	 DEFAULT_CONFIG_FILE);
  printf("-v, --version                 : print the version number\n");
  printf("-H HOSTNAME, --host=HOSTNAME  : on which host is gnuentd running (default: localhost)\n");
}


/**
 * Perform option parsing from the command line. 
 **/
static int parseCommandLine(int argc, 
			    char * argv[],
			    char ** configFile,
			    char ** hostname) {
  int shouldexit=0;
  int c;
  while (1) {
    int option_index=0;
    static struct option long_options[] = {
      { "config",  1, 0, 'c' },
      { "version", 0, 0, 'v' },
      { "help",    0, 0, 'h' },
      { "host",    1, 0, 'H' },
      { 0,0,0,0 }
    };
    
    LOOPPRINT("parseCommandLine"); 
    c = getopt_long(argc,
		    argv, 
		    "vhc:H:", 
		    long_options, 
		    &option_index);
    
    if (c == -1) 
      break;  // No more flags to process
    
    switch(c) {
    case 'H':
      *hostname = optarg;
      break;
    case 'c': 
      *configFile = optarg; 
      MESSAGE("Configfile specified: %s.\n",*configFile);
      break;
    case 'v': 
      printf("GNUnet %s (%s) gnunet-gtk %s\n",
	     GNUNET_VERSION, GNUNET_BRANCH,
	     GNUNET_GTKUI_VERSION);
      shouldexit=1;
      break;
    case 'h': 
      printhelp(); 
      shouldexit=1;
      break;
    default:
      FAILURE("Unknown option %c. Aborting.\nUse --help to get a list of options.\n",
	    c);
      shouldexit=1;    
    } /* end of parsing commandline */
  }
  if (optind < argc) {
    WARNING("Invalid arguments: ");
    while (optind < argc)
      WARNING("%s ", argv[optind++]);
    errexit("Bad arguments.");
  }
  return shouldexit;
}


int main(int argc,
         char * argv[]) {
  char * configfile = DEFAULT_CONFIG_FILE;
  char * hostname = "localhost";

  initLoggingFile(stderr,
		  LOG_MESSAGE,
		  NO);
  parseCommandLine(argc, argv, 
		   &configfile,
		   &hostname);
  /* initialization */
  initConfiguration(configfile);
  initCron();
  initTCPIO();
  initKnownhosts();
  initSearchUtil(hostname,
		 getGNUnetPort());
  
  initRequestManager(hostname);
  initInsertfile(hostname);
  startCron();

  g_thread_init(NULL);
  /*  gdk_threads_init(); */
  gtk_init(&argc, &argv);

  makeMainWindow();
  gdk_threads_enter();
  gtk_main();
  gdk_threads_leave();
 
  return 0;
}
/* end of main.c */
